/* Parser und Interpreter fuer mini-LOGO */

/* Parser -------------------------------------------- */

parse_logo(Logo, Parsebaum):-
  parse_logo1(Logo, Parsebaum, []).

parse_logo1(Liste1, [Befehl|Liste2], Liste4):-
  parse_befehl(Liste1, Befehl, Liste3),
  parse_logo1(Liste3, Liste2, Liste4), !.
parse_logo1(Liste, [], Liste).

parse_befehl([fd, Zahl| Rest], Term, Rest):-
  integer(Zahl),
  Term =.. [fd, Zahl].

parse_befehl([bk|Liste1], bk(Zahl), Liste2):-
  parse_zahl(Liste1, Zahl, Liste2).

parse_befehl([lt|Liste1], lt(Zahl), Liste2):-
  parse_zahl(Liste1, Zahl, Liste2).

parse_befehl([rt|Liste1], rt(Zahl), Liste2):-
  parse_zahl(Liste1, Zahl, Liste2).

parse_befehl([pd|Liste], pd, Liste).

parse_befehl([pu|Liste], pu, Liste).

parse_befehl([rp|Liste1], rp(Zahl, Befehle), Liste3):-
  parse_zahl(Liste1, Zahl, ['['|Liste2]),
  Zahl >= 0,
  parse_logo1(Liste2, Befehle, [']'|Liste3]).
  
parse_zahl(['+', Zahl|Liste], Zahl, Liste):- 
  integer(Zahl), !.
parse_zahl(['-', Zahl1|Liste], Zahl, Liste):-
  integer(Zahl1),
  Zahl is -Zahl1, !.
parse_zahl([Zahl|Liste], Zahl, Liste):-
  integer(Zahl).
  

/* Interpreter --------------------------------- */  

ltest1:- logo([rp, 8, '[', lt, 45, fd, 30, ']']).

ltest2:- logo([rp, 6, '[', rp, 6, '[', fd, 10, lt, 10, ']', lt, 60, rp, 12, '[', fd, 10, lt, 10, ']', lt, 60, ']']).

ltest3:- logo([rp, 8, '[', rp, 8, '[', lt, 45, fd, 30, ']', lt, 45, ']']).

test4:-
  write('Logo-Programm eingeben. Ende mit Strg-D'), nl,
  readln(Eingabe, _, '.', '0123456789', lowercase),
  logo(Eingabe).


logo(Befehle):-
  parse_logo(Befehle, Parsebaum),
  turtle_init,
  asserta(turtle_penpos(down)),
  interpret_logo(Parsebaum).

interpret_logo([Befehl|Befehle]):-
  interpret_befehl(Befehl),
  interpret_logo(Befehle).
interpret_logo([]).
      
/* Forward - fd */
interpret_befehl(fd(Zahl)):-
  turtle_penpos(down),    
  turtle_draw(Zahl).
interpret_befehl(fd(Zahl)):-
  turtle_penpos(up),    
  turtle_move(Zahl).

/* Back - bk */
interpret_befehl(bk(Zahl)):-
  turtle_penpos(down),    
  Zahl1 is -Zahl,
  turtle_draw(Zahl1).
interpret_befehl(bk(Zahl)):-
  turtle_penpos(up),    
  Zahl1 is -Zahl,
  turtle_move(Zahl1).

/* Left - lt */  
interpret_befehl(lt(Zahl)):-
  turtle_turn(Zahl).

/* Right - rt */
interpret_befehl(rt(Zahl)):-
  Zahl1 is -Zahl,
  turtle_turn(Zahl1).

/* Penup - pu */
interpret_befehl(pu):-
  retract(turtle_penpos(_)),
  assert(turtle_penpos(up)).

/* Pendown - pd */  
interpret_befehl(pd):-
  retract(turtle_penpos(_)),
  assert(turtle_penpos(down)).

/* Repeat - rp */
interpret_befehl(rp(0, _Befehle)):- !.
interpret_befehl(rp(Zahl, Befehle)):-
  interpret_logo(Befehle),
  Zahl1 is Zahl - 1,
  interpret_befehl(rp(Zahl1, Befehle)).
